Package: msigdb
Title: An ExperimentHub Package for the Molecular Signatures Database
        (MSigDB)
Version: 1.21.0
Authors@R: c(
            person(given = "Dharmesh D.",
                   family = "Bhuva",
                   role = c("aut", "cre"),
                   email = "bhuva.d@wehi.edu.au",
                   comment = c(ORCID = "0000-0002-6398-9157")),
            person(given = "Gordon K.",
                   family = "Smyth",
                   role = c("aut"),
                   comment = c(ORCID = "0000-0001-9221-2892")),
            person(given = "Alexandra",
                   family = "Garnham",
                   role = c("aut"),
                   comment = c(ORCID = "0000-0002-8312-8450"))
            )
Description: This package provides the Molecular Signatures Database (MSigDB) in
    a R accessible objects. Signatures are stored in GeneSet class objects form
    the GSEABase package and the entire database is stored in a GeneSetCollection
    object. These data are then hosted on the ExperimentHub. Data used in this 
    package was obtained from the MSigDB of the Broad Institute. Metadata for each
    gene set is stored along with the gene set in the GeneSet class object.
biocViews: ExperimentHub, Homo_sapiens_Data, Mus_musculus_Data
License: CC BY 4.0
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.2.3
Depends: R (>= 4.1)
Imports: ExperimentHub, utils, GSEABase, org.Mm.eg.db, org.Hs.eg.db,
        AnnotationDbi, methods, stats, AnnotationHub
Suggests: singscore, vissE, knitr, prettydoc, BiocStyle, rmarkdown,
        testthat (>= 3.0.0), BiocFileCache, GO.db, stringr, limma
URL: https://davislaboratory.github.io/msigdb
BugReports: https://github.com/DavisLaboratory/msigdb/issues
NeedsCompilation: no
VignetteBuilder: knitr
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/msigdb
git_branch: devel
git_last_commit: 1c15606
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-30
Packaged: 2026-04-30 15:57:04 UTC; biocbuild
Author: Dharmesh D. Bhuva [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6398-9157>),
  Gordon K. Smyth [aut] (ORCID: <https://orcid.org/0000-0001-9221-2892>),
  Alexandra Garnham [aut] (ORCID:
    <https://orcid.org/0000-0002-8312-8450>)
Maintainer: Dharmesh D. Bhuva <bhuva.d@wehi.edu.au>
