# Generated by roxygen2: do not edit by hand

export(appendKEGG)
export(getIMEX)
export(getIMEXVersions)
export(getMsigIdType)
export(getMsigOrganism)
export(getMsigdb)
export(getMsigdbIDF)
export(getMsigdbVersions)
export(listCollections)
export(listSubCollections)
export(subsetCollection)
